@@ -14,6 +14,7 @@
 
 
 #include "common.h"
+#include "scrext.h"
 
 #define SALTNAME       "[salt]"          // dummy user name of the salt
 #define ADMINNAME      "admin"           // user name of the administrator
@@ -37,6 +38,8 @@ const char *g_password;                  // path of the password file
 TCMAP *g_users;                          // user list
 const char *g_upload;                    // path of the upload directory
 const char *g_uploadpub;                 // public path of the upload directory
+const char *g_scrext;                    // path of the script extension file
+void *g_scrextproc;                      // processor of the script extension
 int64_t g_recvmax;                       // maximum size of received data
 const char *g_title;                     // site title
 int g_searchnum;                         // number of articles in a search page
@@ -121,6 +124,8 @@ int main(int argc, char **argv){
         }
       }
     }
+    g_scrext = tctmplconf(g_tmpl, "scrext");
+    g_scrextproc = g_scrext && *g_scrext != '\0' ? scrextnew() : NULL;
     const char *rp = tctmplconf(g_tmpl, "recvmax");
     g_recvmax = rp ? tcatoix(rp) : INT_MAX;
     g_title = tctmplconf(g_tmpl, "title");
@@ -152,7 +157,12 @@ int main(int argc, char **argv){
       setenv(name, pv, 1);
       tcfree(name);
     }
+    if(g_scrextproc){
+      scrextsetmapvar(g_scrextproc, "_conf", conf);
+      scrextload(g_scrextproc, g_scrext);
+    }
     eventloop();
+    if(g_scrextproc) scrextdel(g_scrextproc);
     tcfree(uploadpubbuf);
     if(g_users) tcmapdel(g_users);
   } else {
@@ -435,6 +445,28 @@ static void dosession(TCMPOOL *mpool){
   } else if(!strcmp(g_commentmode, "login") && auth){
     cancom = true;
   }
+  // execute the beginning script
+  if(g_scrextproc){
+    const char *emsg = screxterrmsg(g_scrextproc);
+    if(emsg) tclistprintf(emsgs, "Loading scripting extension was failed (%s).", emsg);
+    scrextsetmapvar(g_scrextproc, "_params", params);
+    if(*p_user != '\0'){
+      TCMAP *user = tcmpoolpushmap(mpool, tcmapnew2(TINYBNUM));
+      tcmapput2(user, "name", p_user);
+      tcmapput2(user, "pass", p_pass);
+      if(userinfo) tcmapput2(user, "info", userinfo);
+      scrextsetmapvar(g_scrextproc, "_user", user);
+    }
+  }
+  if(g_scrextproc && scrextcheckfunc(g_scrextproc, "_begin")){
+    char *obuf = tcmpoolpushptr(mpool, scrextcallfunc(g_scrextproc, "_begin", ""));
+    if(obuf){
+      tcmapput2(vars, "beginmsg", obuf);
+    } else {
+      const char *emsg = screxterrmsg(g_scrextproc);
+      tclistprintf(emsgs, "Scripting extension was failed (%s).", emsg ? emsg : "(unknown)");
+    }
+  }
   // open the database
   TCTDB *tdb = tcmpoolpush(mpool, tctdbnew(), (void (*)(void *))tctdbdel);
   int omode = TDBOREADER;
@@ -1177,7 +1209,19 @@ static void dosession(TCMPOOL *mpool){
     }
     tcmapput2(vars, "sidebar", "true");
   }
+  // close the database
   if(!tctdbclose(tdb)) setdberrmsg(emsgs, tdb, "Closing the database was failed.");
+  // execute the ending script
+  if(g_scrextproc && scrextcheckfunc(g_scrextproc, "_end")){
+    char *obuf = tcmpoolpushptr(mpool, scrextcallfunc(g_scrextproc, "_end", ""));
+    if(obuf){
+      tcmapput2(vars, "endmsg", obuf);
+    } else {
+      const char *emsg = screxterrmsg(g_scrextproc);
+      tclistprintf(emsgs, "Scripting extension was failed (%s).", emsg ? emsg : "(unknown)");
+    }
+  }
+  // generete the output
   if(tclistnum(emsgs) > 0) tcmapputlist(vars, "emsgs", emsgs);
   tcmapputmap(vars, "params", params);
   if(tcxstrsize(comquery) > 0) tcmapput2(vars, "comquery", tcxstrptr(comquery));
@@ -1221,6 +1265,10 @@ static void dosession(TCMPOOL *mpool){
   }
   tcmapput2(vars, "mimetype", mtype);
   const char *tmplstr = tcmpoolpushptr(mpool, tctmpldump(g_tmpl, vars));
+  if(g_scrextproc && scrextcheckfunc(g_scrextproc, "_procpage")){
+    char *obuf = tcmpoolpushptr(mpool, scrextcallfunc(g_scrextproc, "_procpage", tmplstr));
+    if(obuf) tmplstr = obuf;
+  }
   printf("Content-Type: %s\r\n", mtype);
   rp = tcmapget2(vars, "lastmod");
   if(rp) printf("Last-Modified: %s\r\n", rp);
@@ -1244,6 +1292,16 @@ static void setdberrmsg(TCLIST *emsgs, TCTDB *tdb, const char *msg){
 
 /* set the HTML data of an article */
 static void setarthtml(TCMPOOL *mpool, TCMAP *cols, int64_t id, int bhl, bool tiny){
+  if(g_scrextproc && scrextcheckfunc(g_scrextproc, "_procart")){
+    TCXSTR *wiki = tcmpoolxstrnew(mpool);
+    wikidump(wiki, cols);
+    char *obuf = tcmpoolpushptr(mpool,
+                                scrextcallfunc(g_scrextproc, "_procart", tcxstrptr(wiki)));
+    if(obuf){
+      tcmapclear(cols);
+      wikiload(cols, obuf);
+    }
+  }
   tcmapprintf(cols, "id", "%lld", (long long)id);
   char idbuf[NUMBUFSIZ];
   sprintf(idbuf, "article%lld", (long long)id);
